import { Graph, Addon, FunctionExt, Shape } from '@antv/x6'
import './shape'
// import graphData from './data'

export default class FlowGraph {


  static init() {
    Graph.registerRouter(//注册自定义默认路由
      'defaultRouter',
      (vertices, args, view) => {
        const points = []
        for (let i = 0; i < 1; i += 1) {
          const sourceCorner = view.sourceBBox.getCenter()
          const targetCorner = view.targetBBox.getCenter()
          const randomPoint = {x:sourceCorner.x,y:targetCorner.y}
          points.push(randomPoint)
        }
        return points
      },
      true
    )
// #region 初始化画布
    this.graph = new Graph({
      container: document.getElementById('flowContainer'),
      width: 8000,
      height: 8000,
    
      minimap: {
        enabled: true,
        width: 200,   
        height: 200, 
        container: document.getElementById('minGraph'),
      },
      grid: {
        // visible: true,
        size: 10,
        type: 'doubleMesh',
        args: [
          {
            color: '#C0D1D3',
            thickness: 0.5
          },
          {
            color: '#B8B8B8',
            thickness: 1,
            factor: 5
          }
        ]
      },
      mousewheel: {
        enabled: true,
        minScale: 0,
        maxScale: 5
      },
      // 选择
      selecting: {
        enabled: true,
        multiple: true,
        rubberband: true,
        movable: true,
        // following:false,
        showNodeSelectionBox: true,
      },
      scroller: {
        enabled: true,
        pageVisible: true,
        pageBreak: true,
        pannable: { enabled: true, eventTypes: ['rightMouseDown'] },
      },
      connecting: {
        snap: {
          radius: 20,
        },
        connectionPoint:'anchor',
        allowBlank: false,
        allowMulti:false,
        highlight: true,
        allowLoop: false,
        allowNode: false,
        allowEdge: false,
        router: {
          name: 'defaultRouter',
        },
        createEdge(){
            return new Shape.Edge({
            attrs: {
              line: {
                stroke: '#000',
                strokeWidth: 1,
                sourceMarker: {
                  tagName: 'circle',
                  r: 1,
                  cx: -1,
                  fill: '#000',
                  stroke: '#000',
                },
                targetMarker: {
                  name: 'circle',
                  r: 1,
                  cx: -1,
                  fill: '#000',
                  stroke: '#000'
                }
              },
            },
            data: {
              shapeType: 'edge'
            },
            router: {
              name: 'defaultRouter'
            },
            zIndex: 0
          })
          // return  this.createEdge({//##通过createEdge创建边时，toJSON（）生成数据时节点属性attrt被丢失
          //   shape: 'default-edge'
          // })
        },
        validateConnection({
          sourceView,
          targetView,
          sourcePort,
          targetPort,
          sourceMagnet,
          targetMagnet,
          sourceCell,
          targetCell
        }) {
          if (sourceView === targetView) {
            return false
          }
          //  只能从输出链接桩创建连接
          if (!sourceMagnet || sourceMagnet.getAttribute('port-group') === 'in') {
            return false
          }
          // 只能连接到输入链接桩
          if (!targetMagnet || targetMagnet.getAttribute('port-group') !== 'in') {
            return false
          }
          return true
        }
      },
      highlighting: {
        magnetAvailable: {
          name: 'stroke',
          args: {
            padding: 4,
            attrs: {
              strokeWidth: 4,
              stroke: 'rgba(223,234,255)'
            }
          }
        }
      },
      snapline: true,
      history: true,
      clipboard: {
        enabled: true
      },
      keyboard: {
        enabled: true
      },
      embedding: {
        enabled: true,
        findParent({ node }) {
          const bbox = node.getBBox()
          return this.getNodes().filter((node) => {
            // 只有 data.parent 为 true 的节点才是父节点
            const data = node.getData()
            if (data && data.parent) {
              const targetBBox = node.getBBox()
              return bbox.isIntersectWithRect(targetBBox)
            }
            return false
          })
        }
      }
    })
  
    this.initEvent()
    return this.graph
  }

  // 初始化画布绘画
  // static initGraphShape() {
  //   this.graph.fromJSON(graphData)
  // }

  // 初始化连接锚点
  static showPorts(ports, show) {
    for (let i = 0, len = ports.length; i < len; i = i + 1) {
      ports[i].style.visibility = show ? 'visible' : 'hidden'
    }
  }

  // 初始化事件
  static initEvent() {
    const { graph } = this

    const container = document.getElementById('flowContainer')
    // graph.on('node:contextmenu', ({ cell, view }) => {
    //   const oldText = cell.attr('text/textWrap/text')
    //   const elem = view.container.querySelector('.x6-edit-text')
    //   if (elem == null) { return }
    //   cell.attr('text/style/display', 'none')
    //   if (elem) {
    //     elem.style.display = ''
    //     elem.contentEditable = 'true'
    //     elem.innerText = oldText
    //     elem.focus()
    //   }
    //   const onBlur = () => {
    //     cell.attr('text/textWrap/text', elem.innerText)
    //     cell.attr('text/style/display', '')
    //     elem.style.display = 'none'
    //     elem.contentEditable = 'false'
    //   }
    //   elem.addEventListener('blur', () => {
    //     onBlur()
    //     elem.removeEventListener('blur', onBlur)
    //   })
    // })
   

    graph.on('node:collapse', ({ node, e }) => {
      e.stopPropagation()
      node.toggleCollapse()
      const collapsed = node.isCollapsed()
      const cells = node.getDescendants()
      cells.forEach((n) => {
        if (collapsed) {
          n.hide()
        } else {
          n.show()
        }
      })
    })

    graph.bindKey('backspace', () => {
      const cells = graph.getSelectedCells()
      if (cells.length) {
        graph.removeCells(cells)
      }
    })
  }

  // 销毁
  static destroy() {
    this.graph.dispose()
  }
}
