// import Vue from 'vue'
import { mapState } from "vuex";
import FlowGraph from '@/graph'
import { forEach, isEmpty } from "lodash";
// import { DagreLayout } from '@antv/layout'
import { Graph, Addon, FunctionExt, Shape } from '@antv/x6'


// const { graph } = FlowGraph

// import { globalGridAttr } from './global'
// import { nodeOpt } from '../components/ConfigPanel/ConfigNode/method'
// import { edgeOpt } from '../components/ConfigPanel/ConfigEdge/method'
const graphMixin = {

    data() {
        return {
            selectedEdgeState: false,
            hoveredEdge: {},
            seletedfirstNodePosition: {},
            pasteToPosition: {},
            isMouseMove: true,
            setMouseTimer: null,
            createdEdge: {}
        }
    },
    mounted() {

    },
    methods: {
        initEvent() {
            let that = this
            const { graph } = FlowGraph
            const { history } = graph
            const container = document.getElementById('flowContainer')
            const svgEle = $('.x6-graph-svg')[0]
            let source = {}, target = {}
            
            // 生成符合连线模板的边
            function buildEdgeByCurrentEdgeTemplate(edge) {
                let deviceTemplateConfig = JSON.parse(that.currentEdgeTemplate.deviceTemplateConfig)
                edge.attr('line', deviceTemplateConfig.attrs.line);
                edge.data = {
                    id: that.currentEdgeTemplate.id ? that.currentEdgeTemplate.id : (() => { return that.$message.error('未获取到连接模板Id!') })(),
                    deviceModelId: that.currentEdgeTemplate.deviceModelId ? that.currentEdgeTemplate.deviceModelId : (() => { return that.$message.error('未获取到型号Id!') })(),
                    cableLength: deviceTemplateConfig['data']['cableLength'] ? deviceTemplateConfig['data']['cableLength'] : (() => { return that.$message.error('未获取到连线cableLength，请查看模板配置!') })(),
                    shapeType: 'edge'
                }
                return edge
            }

            function showPorts(ports, show) {
                for (let i = 0, len = ports.length; i < len; i = i + 1) {
                    ports[i].style.visibility = show ? 'visible' : 'hidden'
                }
            }
            // 添加边
            function addEdge() {
                if (isEmpty(source) || isEmpty(target)) return
                let edge = graph.addEdge({
                    source: source,
                    target: target
                })
                source = {}, target = {}//完成当前连线，源节点和目标节点置空

                return buildEdgeByCurrentEdgeTemplate(edge)
            }
            function removeEdge() {
                if (!isEmpty(that.createdEdge)) graph.removeCell(that.createdEdge.id)
                source = {}
                target = {}
            }
          
            // 检查链接桩是否被使用
            function checkSelectedPort(id, port) {
                let edges = graph.getConnectedEdges(id)
                let selectPort = edges.find((ele) => { return (ele.source.cell == id && ele.source.port == port) || (ele.target.cell == id && ele.target.port == port) })
                if (selectPort) {//该节点的对应链接桩已被连接
                    source = {}
                    target = {}
                    return false
                }
                return true
            }
            // 点击链接桩
            graph.on("port:click", (e) => {
                let id = e.cell.id
                let port = e.e.target.attributes[3].value
                let portType = (e.cell.ports.items.find((ele) => ele.id == port)).group

                if (!checkSelectedPort(id, port)) return false
                switch (portType) {
                    case 'out':
                        source = {}, target = {}
                        source = {
                            cell: id,
                            port: port
                        }
                        this.isMouseMove = false
                        break
                    case 'in':
                        target = {
                            cell: id,
                            port: port
                        }
                        this.isMouseMove = true
                        break
                }
                that.createdEdge = addEdge()
            });
            graph.on('edge:connected', ({ isNew, edge }) => {

                if (isNew) {
                    let currentEdge = buildEdgeByCurrentEdgeTemplate(edge)
                    //标注连接状态
                    const source = currentEdge.getSourceCell()
                    const target = currentEdge.getTargetCell()
                    source['data']['connectStatus'] = true
                    target['data']['connectStatus'] = true
                }
            })
            graph.on('edge:removed', ({ edge, index, options }) => {
                const sourceId = edge.getSourceCellId()
                const targetId = edge.getTargetCellId()
                const source = graph.getCellById(sourceId)
                const target = graph.getCellById(targetId)
                if (source) source['data']['connectStatus'] = false
                if (target) target['data']['connectStatus'] = false
            })
            function createEdgeByMousemove(e) {
                if (that.isMouseMove||e.toElement.tagName == 'path') return
                if (!isEmpty(that.createdEdge)) graph.removeCell(that.createdEdge.id)

                let zoom = (graph.zoom()).toFixed(4)
                target = { x: parseInt(e.layerX / zoom - 10), y: parseInt(e.layerY / zoom) }
                that.createdEdge = addEdge()

            }
            // svgEle.addEventListener('mousemove', createEdgeByMousemove)//画布监听鼠标移动事件 性能不好舍弃
            graph.on(
                'node:mouseenter',
                FunctionExt.debounce(() => {
                    const ports = container.querySelectorAll(
                        '.x6-port-body'
                    )
                    showPorts(ports, true)
                }),
                500
            )
            graph.on('node:mouseleave', () => {
                const ports = container.querySelectorAll(
                    '.x6-port-body'
                )
                showPorts(ports, false)
            })
            history.on('change', () => {
                this.canUndo = history.canUndo()
                this.canRedo = history.canRedo()
            })
            graph.bindKey('ctrl+z', () => {
                if (history.canUndo()) {
                    history.undo()
                }
                return false
            })
            graph.bindKey('ctrl+shift+z', () => {
                if (history.canRedo()) {
                    history.redo()
                }
                return false
            })
            graph.bindKey('ctrl+d', () => {
                graph.clearCells()
                return false
            })
            graph.bindKey('ctrl+s', () => {
                graph.toPNG((datauri) => {
                    DataUri.downloadDataUri(datauri, 'chart.png')
                })
                return false
            })
            graph.bindKey('ctrl+p', () => {
                graph.printPreview()
                return false
            })
            graph.bindKey('ctrl+c', this.copy)
            graph.bindKey('ctrl+v', this.paste)
            graph.bindKey('ctrl+x', this.cut)
            graph.bindKey('delete', this.delete)
            graph.on('blank:click', ({ e, x, y }) => {
                this.pasteToPosition = { x: x, y: y }
            })
            graph.on('selection:changed', (args) => {
                args.added.forEach((cell) => {
                    if (!cell.isNode()) {
                        // cell.addTools('vertices', 'onhover')
                        cell.attr('line/stroke', '#feb663')
                        cell.attr('line/strokeWidth', 5)

                    }
                })
                args.removed.forEach((cell) => {
                    if (cell['data']['id']) {
                        let id = cell['data']['id']
                        let deviceTemplateConfig = JSON.parse(that.allCellTemplate[id].deviceTemplateConfig)
                        if (!cell.isNode()) {
                            cell.attr('line', deviceTemplateConfig['attrs']['line'])
                            if (cell.hasTools('onhover')) {
                                // cell.removeTools()
                            }
                        }
                    }
                })
            })
            // graph.on('edge:mouseenter', ({ cell }) => {
            //         // cell.attr('line/strokeWidth', 5)
            //         // cell.attr('line/stroke', '#feb663')
            //     
            // })
            // graph.on('edge:mouseleave', ({ cell }) => {
            //      let id = cell['data']['id']
            //      let deviceTemplateConfig = JSON.parse(this.allCellTemplate[id].deviceTemplateConfig)
            //       cell.attr('line', deviceTemplateConfig['attrs']['line'])
            // })

            graph.on('cell:dblclick', ({ cell }) => {
                that.type = cell.isNode() ? 'node' : 'edge'
                that.id = cell.id
                that.$refs.cellAttrModal.edit(cell, that.type)
            })
        
        },
        delete() {
            const { graph } = FlowGraph
            const cells = graph.getSelectedCells()
            if (cells.length) {
                graph.removeCells(cells)
            }
            return false
        },
        copy() {

            const { graph } = FlowGraph
            const cells = graph.getSelectedCells()

            this.seletedfirstNodePosition = cells[0].position()
            if (cells.length) {
                graph.copy(cells)
            }
            return false
        },
        cut() {
            const { graph } = FlowGraph
            const cells = graph.getSelectedCells()
            if (cells.length) {
                graph.cut(cells)
            }
            return false
        },
        paste() {
            const { graph } = FlowGraph
            if (!graph.isClipboardEmpty()) {
                let x = this.pasteToPosition.x - this.seletedfirstNodePosition.x
                let y = this.pasteToPosition.y - this.seletedfirstNodePosition.y
                const cells = graph.paste({ useLocalStorage: false, offset: { dx: x || 150, dy: y || 150 } })
                graph.cleanSelection()
                graph.select(cells)
                graph.cleanSelection()
                this.pasteToPosition = {}
                this.seletedfirstNodePosition = {}
            }

            return false
        },

        toolHandleClick(event) {
            const { graph } = FlowGraph
            const name = event.currentTarget.name
            switch (name) {
                case 'undo':
                    graph.history.undo()
                    break
                case 'redo':
                    graph.history.redo()
                    break
                case 'delete':
                    this.$confirm({
                        content: '确定清除当前画板的拓扑图吗',
                        mask: false,
                        okText: '是',
                        cancelText: '否',
                        onOk() {
                            graph.clearCells()
                            graph.centerContent({ padding: { left: -6200, top: -6200 } })
                        },
                    })
                    break
                case 'savePNG':
                    graph.toPNG(
                        (dataUri) => {
                            // 下载
                            DataUri.downloadDataUri(dataUri, 'chartx.png')
                        },
                        {
                            backgroundColor: 'white',
                            padding: {
                                top: 20,
                                right: 30,
                                bottom: 40,
                                left: 50,
                            },
                            quality: 1,
                        }
                    )
                    break
                case 'saveSVG':
                    graph.toSVG((dataUri) => {
                        // 下载
                        DataUri.downloadDataUri(DataUri.svgToDataUrl(dataUri), 'chart.svg')
                    })
                    break
                case 'print':
                    graph.printPreview()
                    break
                case 'copy':
                    this.copy()
                    break
                case 'cut':
                    this.cut()
                    break
                case 'paste':
                    this.paste()
                    break
                case 'toJSON':
                    console.log(graph.toJSON())
                    break
                case 'alignGraph':
                    this.graphDagreLayout()
                    break
                case 'contextMenu':
                    this.contextMenuState = true
                    break
                default:
                    break
            }
        },
        // // 图对齐   
        // graphDagreLayout() {
        //     const { graph } = FlowGraph
        //     let graphData = graph.toJSON().cells
        //     let edges = [], nodes = []
        //     for (let index = 0; index < graphData.length; index++) {
        //         const ele = graphData[index];
        //         let type = ele.data['shapeType']
        //         switch (type) {
        //             case 'edge':
        //                 edges.push(ele)
        //                 break
        //             case 'node':
        //                 nodes.push(ele)
        //                 break
        //         }
        //     }
        //     const dagreLayout = new DagreLayout({
        //         type: 'dagre',
        //         rankdir: 'LR',
        //         align: 'UR',
        //         ranksep: 30,
        //         nodesep: 30,
        //         nodeSize: 60
        //     })
        //     let mode = dagreLayout.layout({ nodes: nodes, edges: edges })
        //     graph.fromJSON(mode)
        //     this.alignGraph()
        // },

        alignGraph() {
            const { graph } = FlowGraph
            let graphData = graph.toJSON().cells
            let edges = [], nodeMap = {}, nodes = []
            for (let index = 0; index < graphData.length; index++) {
                const ele = graphData[index];
                let type = ele.data['shapeType']
                switch (type) {
                    case 'edge':
                        edges.push(ele)
                        break
                    case 'node':
                        nodeMap[ele.id] = ele
                        nodes.push(ele)
                        break
                }
            }
            edges.forEach((one) => {
                let sourceId = one['source']['cell']
                let sourcePort = one['source']['port']
                let targetId = one['target']['cell']
                let targetPort = one['target']['port']

                if (nodeMap[sourceId]['data']['deviceModelId'] == 14) sourcePort = sourcePort - 1
                let sourcePortItem = nodeMap[sourceId]['ports']['items'][sourcePort]
                let targetPortItem = nodeMap[targetId]['ports']['items'][targetPort]
                let x = nodeMap[sourceId]['position']['x'] + sourcePortItem['args']['x']
                let y = nodeMap[targetId]['position']['y'] + targetPortItem['args']['y']
                delete one.router
                one['vertices'] = [
                    { x: x, y: y }
                ]
            })
            const dagreLayout = new DagreLayout({
                type: 'dagre',
                rankdir: 'LR',
                align: 'UR',
                ranksep: 30,
                nodesep: 30,
                nodeSize: 60
            })
            let mode = dagreLayout.layout({ nodes: nodes, edges: edges })
            graph.fromJSON(mode)
        },
    }
}

export { graphMixin }